# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.source_name import SourceName
from typing import Optional, Set
from typing_extensions import Self

class CdmGetUserRequest(BaseModel):
    """
    Get details of users. Wildcard without filters is disabled. At most 1 filter can be specified.
    """ # noqa: E501
    user_details: Optional[StrictBool] = Field(default=None, description="Return the entire user details. By default this value is false, and only the firstname, lastname, email, ext_org_id, and IDs are returned.")
    filter_organisation_id: Optional[StrictStr] = Field(default=None, description="Only include users belonging to the specified organisation.")
    filter_group_id: Optional[StrictStr] = Field(default=None, description="Only include users with the specified group.")
    filter_organisation_ext_identifier: Optional[StrictStr] = Field(default=None, description="Only include users belonging to the organisation with the specified external identifier.")
    filter_ext_ids: Optional[List[StrictStr]] = Field(default=None, description="Only include users with the specified user external IDs. This filter must be used in conjunction with filter_ext_source.")
    filter_ext_source: Optional[SourceName] = SourceName.INVALID_SOURCE_NAME
    user_ids: Optional[List[StrictStr]] = Field(default=None, description="User IDs (optional, default wildcard for all IDs user can access). If specified, the request is not a wildcard and users are returned in the order that their IDs were specified. Otherwise, if not specified, the request is considered a wildcard and users are returned in the order that they were created.")
    bookmark: Optional[StrictStr] = Field(default=None, description="Opaque pagination bookmark. Set to the empty string (or omit) to begin.")
    filter_emails: Optional[List[StrictStr]] = Field(default=None, description="Only include users with the specified emails.")
    page_size: Optional[StrictInt] = Field(default=None, description="Specify a limit on the number of items returned in a page. Values that are too large will be rejected. All requests are subject to the page size, which is 100 if not specified and can be specified up to 500.")
    __properties: ClassVar[List[str]] = ["user_details", "filter_organisation_id", "filter_group_id", "filter_organisation_ext_identifier", "filter_ext_ids", "filter_ext_source", "user_ids", "bookmark", "filter_emails", "page_size"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmGetUserRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmGetUserRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "user_details": obj.get("user_details"),
            "filter_organisation_id": obj.get("filter_organisation_id"),
            "filter_group_id": obj.get("filter_group_id"),
            "filter_organisation_ext_identifier": obj.get("filter_organisation_ext_identifier"),
            "filter_ext_ids": obj.get("filter_ext_ids"),
            "filter_ext_source": obj.get("filter_ext_source") if obj.get("filter_ext_source") is not None else SourceName.INVALID_SOURCE_NAME,
            "user_ids": obj.get("user_ids"),
            "bookmark": obj.get("bookmark"),
            "filter_emails": obj.get("filter_emails"),
            "page_size": obj.get("page_size")
        })
        return _obj


