# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.cdm_case_details import CdmCaseDetails
from acre_api.models.cdm_client_details import CdmClientDetails
from acre_api.models.cdm_property_details import CdmPropertyDetails
from acre_api.models.cdm_user import CdmUser
from acre_api.models.hydrated_case_mortgage_pair import HydratedCaseMortgagePair
from typing import Optional, Set
from typing_extensions import Self

class CdmHydratedCase(BaseModel):
    """
    HydratedCase is a convenience wrapper for a case, its clients and all properties and mortgages associated with them. It also includes any donors and the case owner user. This is used by microservices to load an entire case quickly and is for internal use only.
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="The ID of the case.")
    case_details: Optional[CdmCaseDetails] = None
    clients: Optional[List[CdmClientDetails]] = Field(default=None, description="The clients for the case.")
    mortgages: Optional[List[HydratedCaseMortgagePair]] = Field(default=None, description="The requested mortgages as mortgage-mortgageproduct pairs.")
    properties: Optional[List[CdmPropertyDetails]] = Field(default=None, description="The requested properties.")
    donors: Optional[List[CdmClientDetails]] = Field(default=None, description="The donors for the case if requested.")
    case_owner: Optional[CdmUser] = None
    __properties: ClassVar[List[str]] = ["id", "case_details", "clients", "mortgages", "properties", "donors", "case_owner"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmHydratedCase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of case_details
        if self.case_details:
            _dict['case_details'] = self.case_details.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in clients (list)
        _items = []
        if self.clients:
            for _item_clients in self.clients:
                if _item_clients:
                    _items.append(_item_clients.to_dict())
            _dict['clients'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in mortgages (list)
        _items = []
        if self.mortgages:
            for _item_mortgages in self.mortgages:
                if _item_mortgages:
                    _items.append(_item_mortgages.to_dict())
            _dict['mortgages'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in properties (list)
        _items = []
        if self.properties:
            for _item_properties in self.properties:
                if _item_properties:
                    _items.append(_item_properties.to_dict())
            _dict['properties'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in donors (list)
        _items = []
        if self.donors:
            for _item_donors in self.donors:
                if _item_donors:
                    _items.append(_item_donors.to_dict())
            _dict['donors'] = _items
        # override the default output from pydantic by calling `to_dict()` of case_owner
        if self.case_owner:
            _dict['case_owner'] = self.case_owner.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmHydratedCase from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "case_details": CdmCaseDetails.from_dict(obj["case_details"]) if obj.get("case_details") is not None else None,
            "clients": [CdmClientDetails.from_dict(_item) for _item in obj["clients"]] if obj.get("clients") is not None else None,
            "mortgages": [HydratedCaseMortgagePair.from_dict(_item) for _item in obj["mortgages"]] if obj.get("mortgages") is not None else None,
            "properties": [CdmPropertyDetails.from_dict(_item) for _item in obj["properties"]] if obj.get("properties") is not None else None,
            "donors": [CdmClientDetails.from_dict(_item) for _item in obj["donors"]] if obj.get("donors") is not None else None,
            "case_owner": CdmUser.from_dict(obj["case_owner"]) if obj.get("case_owner") is not None else None
        })
        return _obj


