# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.cdm_asu import CdmAsu
from acre_api.models.cdm_equity_release_lifetime import CdmEquityReleaseLifetime
from acre_api.models.cdm_fee_configuration import CdmFeeConfiguration
from acre_api.models.cdm_gi import CdmGi
from acre_api.models.cdm_pmi import CdmPmi
from acre_api.models.cdm_protection import CdmProtection
from acre_api.models.panel_management_panel import PanelManagementPanel
from typing import Optional, Set
from typing_extensions import Self

class CdmPanelManagement(BaseModel):
    """
    PanelManagement manages panel related information.
    """ # noqa: E501
    panel: Optional[PanelManagementPanel] = PanelManagementPanel.INVALID_LENDER_PANEL_TYPE
    excluded_lenders: Optional[List[StrictStr]] = Field(default=None, description="Lender Organisation IDs excluded at (broker) Organisation level.")
    preferred_clubs: Optional[List[StrictStr]] = Field(default=None, description="Club Organisation IDs of which the Organisation is a member of.")
    default_club: Optional[StrictStr] = Field(default=None, description="Default Club of the Organisation.")
    fee_configurations: Optional[List[CdmFeeConfiguration]] = Field(default=None, description="Fee Configuration holds configuration when to generate certain fee for certain lenders.")
    mortgage_advice: Optional[StrictBool] = Field(default=None, description="Whether the organisation offers mortgage advice.")
    second_charge_advice: Optional[StrictBool] = Field(default=None, description="Whether the organisation offers second charge advice.")
    bridging_advice: Optional[StrictBool] = Field(default=None, description="Whether the organisation offers bridging advice.")
    commercial_advice: Optional[StrictBool] = Field(default=None, description="Whether the organisation offers commercial advice.")
    equity_release_lifetime: Optional[CdmEquityReleaseLifetime] = None
    protection: Optional[CdmProtection] = None
    pmi: Optional[CdmPmi] = None
    asu: Optional[CdmAsu] = None
    gi: Optional[CdmGi] = None
    __properties: ClassVar[List[str]] = ["panel", "excluded_lenders", "preferred_clubs", "default_club", "fee_configurations", "mortgage_advice", "second_charge_advice", "bridging_advice", "commercial_advice", "equity_release_lifetime", "protection", "pmi", "asu", "gi"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmPanelManagement from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in fee_configurations (list)
        _items = []
        if self.fee_configurations:
            for _item_fee_configurations in self.fee_configurations:
                if _item_fee_configurations:
                    _items.append(_item_fee_configurations.to_dict())
            _dict['fee_configurations'] = _items
        # override the default output from pydantic by calling `to_dict()` of equity_release_lifetime
        if self.equity_release_lifetime:
            _dict['equity_release_lifetime'] = self.equity_release_lifetime.to_dict()
        # override the default output from pydantic by calling `to_dict()` of protection
        if self.protection:
            _dict['protection'] = self.protection.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pmi
        if self.pmi:
            _dict['pmi'] = self.pmi.to_dict()
        # override the default output from pydantic by calling `to_dict()` of asu
        if self.asu:
            _dict['asu'] = self.asu.to_dict()
        # override the default output from pydantic by calling `to_dict()` of gi
        if self.gi:
            _dict['gi'] = self.gi.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmPanelManagement from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "panel": obj.get("panel") if obj.get("panel") is not None else PanelManagementPanel.INVALID_LENDER_PANEL_TYPE,
            "excluded_lenders": obj.get("excluded_lenders"),
            "preferred_clubs": obj.get("preferred_clubs"),
            "default_club": obj.get("default_club"),
            "fee_configurations": [CdmFeeConfiguration.from_dict(_item) for _item in obj["fee_configurations"]] if obj.get("fee_configurations") is not None else None,
            "mortgage_advice": obj.get("mortgage_advice"),
            "second_charge_advice": obj.get("second_charge_advice"),
            "bridging_advice": obj.get("bridging_advice"),
            "commercial_advice": obj.get("commercial_advice"),
            "equity_release_lifetime": CdmEquityReleaseLifetime.from_dict(obj["equity_release_lifetime"]) if obj.get("equity_release_lifetime") is not None else None,
            "protection": CdmProtection.from_dict(obj["protection"]) if obj.get("protection") is not None else None,
            "pmi": CdmPmi.from_dict(obj["pmi"]) if obj.get("pmi") is not None else None,
            "asu": CdmAsu.from_dict(obj["asu"]) if obj.get("asu") is not None else None,
            "gi": CdmGi.from_dict(obj["gi"]) if obj.get("gi") is not None else None
        })
        return _obj


