# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.case_details_mortgage_reason import CaseDetailsMortgageReason
from acre_api.models.cdm_auto_satisfy import CdmAutoSatisfy
from acre_api.models.cdm_case_details_status import CdmCaseDetailsStatus
from acre_api.models.cdm_product_type import CdmProductType
from acre_api.models.cdm_requirement_details_status import CdmRequirementDetailsStatus
from acre_api.models.document_document_type import DocumentDocumentType
from typing import Optional, Set
from typing_extensions import Self

class CdmRequirementDetails(BaseModel):
    """
    The details of the requirement.
    """ # noqa: E501
    organisation_id: Optional[StrictStr] = Field(default=None, description="Identifier of the owning organisation.")
    name: Optional[StrictStr] = Field(default=None, description="Name of the requirement.")
    description: Optional[StrictStr] = Field(default=None, description="Description of the requirement.")
    status: Optional[CdmRequirementDetailsStatus] = CdmRequirementDetailsStatus.OPTIONAL
    gi: Optional[StrictBool] = Field(default=None, description="Whether or not to enforce over general insurance products.")
    protection_types: Optional[List[CdmProductType]] = Field(default=None, description="List of product types to run this requirement on.")
    case_types: Optional[List[CaseDetailsMortgageReason]] = Field(default=None, description="Case types to which this requirement applies.")
    document_types: Optional[List[DocumentDocumentType]] = Field(default=None, description="Array of document types that are accepted for this requirement.")
    auto_satisfy: Optional[List[CdmAutoSatisfy]] = Field(default=None, description="Document types and count pairings to determine when to automatically satisfy this requirement.")
    default_overdue: Optional[StrictInt] = Field(default=None, description="Number of days for requirement to be considered overdue.")
    per_client: Optional[StrictBool] = Field(default=None, description="If more than one client on a case, whether this requirement is per client.")
    can_share_with_client: Optional[StrictBool] = Field(default=None, description="If true, will show if the client is the caller.")
    required_from: Optional[CdmCaseDetailsStatus] = CdmCaseDetailsStatus.INVALID_STATUS
    required_by: Optional[CdmCaseDetailsStatus] = CdmCaseDetailsStatus.INVALID_STATUS
    apply_to_existing: Optional[StrictBool] = Field(default=None, description="Whether to apply over existing cases.")
    __properties: ClassVar[List[str]] = ["organisation_id", "name", "description", "status", "gi", "protection_types", "case_types", "document_types", "auto_satisfy", "default_overdue", "per_client", "can_share_with_client", "required_from", "required_by", "apply_to_existing"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmRequirementDetails from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in auto_satisfy (list)
        _items = []
        if self.auto_satisfy:
            for _item_auto_satisfy in self.auto_satisfy:
                if _item_auto_satisfy:
                    _items.append(_item_auto_satisfy.to_dict())
            _dict['auto_satisfy'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmRequirementDetails from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "organisation_id": obj.get("organisation_id"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "status": obj.get("status") if obj.get("status") is not None else CdmRequirementDetailsStatus.OPTIONAL,
            "gi": obj.get("gi"),
            "protection_types": obj.get("protection_types"),
            "case_types": obj.get("case_types"),
            "document_types": obj.get("document_types"),
            "auto_satisfy": [CdmAutoSatisfy.from_dict(_item) for _item in obj["auto_satisfy"]] if obj.get("auto_satisfy") is not None else None,
            "default_overdue": obj.get("default_overdue"),
            "per_client": obj.get("per_client"),
            "can_share_with_client": obj.get("can_share_with_client"),
            "required_from": obj.get("required_from") if obj.get("required_from") is not None else CdmCaseDetailsStatus.INVALID_STATUS,
            "required_by": obj.get("required_by") if obj.get("required_by") is not None else CdmCaseDetailsStatus.INVALID_STATUS,
            "apply_to_existing": obj.get("apply_to_existing")
        })
        return _obj


