# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.cdm_export_type import CdmExportType
from acre_api.models.cdm_file_type import CdmFileType
from acre_api.models.cdm_template_name import CdmTemplateName
from typing import Optional, Set
from typing_extensions import Self

class CdmTemplate(BaseModel):
    """
    Templates are used to generate parameterised documents.
    """ # noqa: E501
    template_id: Optional[StrictStr] = Field(default=None, description="ID of the template (auto-populated).")
    owning_organisation_id: Optional[StrictStr] = Field(default=None, description="Acre ID of organisation (optional, defaults to user's organisation).")
    date_stored: Optional[StrictStr] = Field(default=None, description="When this template was uploaded (RFC3339, auto-populated).")
    name: Optional[CdmTemplateName] = CdmTemplateName.INVALID_TEMPLATE_NAME
    template: Optional[StrictStr] = Field(default=None, description="Base64 encoded template (required).")
    template_signature: Optional[StrictStr] = Field(default=None, description="Message digest (SHA-512) of template file (auto-populated).")
    confidential: Optional[StrictBool] = None
    allow_child_access: Optional[StrictBool] = Field(default=None, description="Can organisations that are administered by or regulated by access this template.")
    enforce_for_representative: Optional[StrictBool] = Field(default=None, description="If a child is a representative, enforce use of this template.")
    updated_by: Optional[StrictStr] = Field(default=None, description="User reference (UUID) of individual who updated this template last (auto-populated).")
    required_organisations: Optional[StrictInt] = Field(default=None, description="Number of organisations required to render the template.")
    required_users: Optional[StrictInt] = Field(default=None, description="Number of users required to render the template.")
    required_clients: Optional[StrictInt] = Field(default=None, description="Number of clients required to render the template.")
    required_cases: Optional[StrictInt] = Field(default=None, description="Number of cases required to render the template.")
    required_mortgages: Optional[StrictInt] = Field(default=None, description="Number of mortgages required to render the template.")
    required_products: Optional[StrictInt] = Field(default=None, description="Number of products required to render the template.")
    required_notes: Optional[StrictInt] = Field(default=None, description="Number of notes required to render the template.")
    file_type: Optional[CdmFileType] = CdmFileType.INVALID_FILE_TYPE
    render_to: Optional[CdmExportType] = CdmExportType.INVALID_EXPORT_TYPE
    disabled: Optional[StrictBool] = Field(default=None, description="Whether the template is disabled. Not yet implemented.")
    sub_type: Optional[StrictStr] = Field(default=None, description="The template sub_type. Must be in SNAKE_CASE format.")
    __properties: ClassVar[List[str]] = ["template_id", "owning_organisation_id", "date_stored", "name", "template", "template_signature", "confidential", "allow_child_access", "enforce_for_representative", "updated_by", "required_organisations", "required_users", "required_clients", "required_cases", "required_mortgages", "required_products", "required_notes", "file_type", "render_to", "disabled", "sub_type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmTemplate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmTemplate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "template_id": obj.get("template_id"),
            "owning_organisation_id": obj.get("owning_organisation_id"),
            "date_stored": obj.get("date_stored"),
            "name": obj.get("name") if obj.get("name") is not None else CdmTemplateName.INVALID_TEMPLATE_NAME,
            "template": obj.get("template"),
            "template_signature": obj.get("template_signature"),
            "confidential": obj.get("confidential"),
            "allow_child_access": obj.get("allow_child_access"),
            "enforce_for_representative": obj.get("enforce_for_representative"),
            "updated_by": obj.get("updated_by"),
            "required_organisations": obj.get("required_organisations"),
            "required_users": obj.get("required_users"),
            "required_clients": obj.get("required_clients"),
            "required_cases": obj.get("required_cases"),
            "required_mortgages": obj.get("required_mortgages"),
            "required_products": obj.get("required_products"),
            "required_notes": obj.get("required_notes"),
            "file_type": obj.get("file_type") if obj.get("file_type") is not None else CdmFileType.INVALID_FILE_TYPE,
            "render_to": obj.get("render_to") if obj.get("render_to") is not None else CdmExportType.INVALID_EXPORT_TYPE,
            "disabled": obj.get("disabled"),
            "sub_type": obj.get("sub_type")
        })
        return _obj


