# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.cdm_auto_satisfy import CdmAutoSatisfy
from acre_api.models.cdm_entity import CdmEntity
from acre_api.models.cdm_verification_class import CdmVerificationClass
from acre_api.models.cdm_verification_scope import CdmVerificationScope
from acre_api.models.verification_flag_status import VerificationFlagStatus
from typing import Optional, Set
from typing_extensions import Self

class CdmVerification(BaseModel):
    """
    Verification indicates a gate or test that was conducted on a case, and the results of that test. Verification failures accumulate, so that the UI can see everything that is failing. Validations will short circuit such that if a parent field is missing, then the backend will not perform the subsequnet validations on child entities.
    """ # noqa: E501
    verification_id: Optional[StrictStr] = Field(default=None, description="ID of the test.")
    name: Optional[StrictStr] = Field(default=None, description="Name of the test.")
    passed: Optional[StrictBool] = Field(default=None, description="Whether or not the test passed.")
    explanation: Optional[StrictStr] = Field(default=None, description="An explanation of the test.")
    case_id: Optional[StrictStr] = Field(default=None, description="Case ID, if case was the target of the verification.")
    client_id: Optional[StrictStr] = Field(default=None, description="Client ID, if client was the target of the verification.")
    mortgage_id: Optional[StrictStr] = Field(default=None, description="Mortgage ID, if mortgage was the target of the verification.")
    product_id: Optional[StrictStr] = Field(default=None, description="Product ID, if product was the target of the verification.")
    property_id: Optional[StrictStr] = Field(default=None, description="Property ID, if property was the target of the verification.")
    failed_fields: Optional[List[StrictStr]] = Field(default=None, description="Fields that failed validation.")
    flag: Optional[VerificationFlagStatus] = VerificationFlagStatus.INVALID_FLAG_STATUS
    general_insurance_product_id: Optional[StrictStr] = Field(default=None, description="General insurance product ID, if a GI product was the target of the verification.")
    classes: Optional[List[CdmVerificationClass]] = Field(default=None, description="Classes that the verification belongs to.")
    optional: Optional[StrictBool] = Field(default=None, description="Indicates whether the verification is optional.")
    iterable_id: Optional[StrictStr] = Field(default=None, description="Unique ID of the entity being iterated over in the verification.")
    entities: Optional[List[CdmEntity]] = Field(default=None, description="Entities that were tested in the verification.")
    scopes: Optional[List[CdmVerificationScope]] = Field(default=None, description="Scopes that the verification belongs to.")
    auto_satisfy: Optional[List[CdmAutoSatisfy]] = Field(default=None, description="Auto satisfy field from document requirements.")
    __properties: ClassVar[List[str]] = ["verification_id", "name", "passed", "explanation", "case_id", "client_id", "mortgage_id", "product_id", "property_id", "failed_fields", "flag", "general_insurance_product_id", "classes", "optional", "iterable_id", "entities", "scopes", "auto_satisfy"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmVerification from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in entities (list)
        _items = []
        if self.entities:
            for _item_entities in self.entities:
                if _item_entities:
                    _items.append(_item_entities.to_dict())
            _dict['entities'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in auto_satisfy (list)
        _items = []
        if self.auto_satisfy:
            for _item_auto_satisfy in self.auto_satisfy:
                if _item_auto_satisfy:
                    _items.append(_item_auto_satisfy.to_dict())
            _dict['auto_satisfy'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmVerification from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "verification_id": obj.get("verification_id"),
            "name": obj.get("name"),
            "passed": obj.get("passed"),
            "explanation": obj.get("explanation"),
            "case_id": obj.get("case_id"),
            "client_id": obj.get("client_id"),
            "mortgage_id": obj.get("mortgage_id"),
            "product_id": obj.get("product_id"),
            "property_id": obj.get("property_id"),
            "failed_fields": obj.get("failed_fields"),
            "flag": obj.get("flag") if obj.get("flag") is not None else VerificationFlagStatus.INVALID_FLAG_STATUS,
            "general_insurance_product_id": obj.get("general_insurance_product_id"),
            "classes": obj.get("classes"),
            "optional": obj.get("optional"),
            "iterable_id": obj.get("iterable_id"),
            "entities": [CdmEntity.from_dict(_item) for _item in obj["entities"]] if obj.get("entities") is not None else None,
            "scopes": obj.get("scopes"),
            "auto_satisfy": [CdmAutoSatisfy.from_dict(_item) for _item in obj["auto_satisfy"]] if obj.get("auto_satisfy") is not None else None
        })
        return _obj


