# coding: utf-8

"""
    Acre Platform User API

    # CORS CORS is implemented by all Acre API endpoints and only certain hostnames are allowed for each deployment of the API. If your use case requires calling the Acre User API from a browser, please contact Acre to arrange the appropriate configuration. # API Authentication The primary method of authentication for users of the Acre API is with the use of JSON Web Tokens (JWTs), which are suplemented by an application specific API Key. The API key must be provided through the `X-API-KEY` HTTP header and will be provided by your Acre Customer Success Manager. The JWT must be provided through a cookie with key of `authorization`. The JWT must be generated and signed as per the following section. # Generating JWTs JWT must either be generated by a third-party such as Azure ADB2C that has been configured for the appropriate service and/or user account, or alternatively can be generated locally and signed by an private key for which the public key has been made available to Acre via a JWKS URL. In addition to being signed by your private key, the JWT will also need to contain at a minimum:  * An issuer `iss`, which must be a valid URI previously agreed with Acre.  * A subject `sub`, which must be a unique identifier for the user and/or service account previously agreed with Acre.  * An expiry time `exp` which must not be more than four hours in the future.  * A not before time `nbf` which must be in the past, but not greater than six hours in the past.  The JWT should be signed using RS256.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from acre_api.models.cdm_verification import CdmVerification
from typing import Optional, Set
from typing_extensions import Self

class CdmVerificationCollection(BaseModel):
    """
    VerificationCollection aggregate verifications (e.g., Gates).
    """ # noqa: E501
    collection_id: Optional[StrictStr] = Field(default=None, description="ID of the verification collection.")
    name: Optional[StrictStr] = None
    passed: Optional[StrictBool] = Field(default=None, description="Whether or not the collection of verifications passed.")
    verifications: Optional[List[CdmVerification]] = Field(default=None, description="The verifications within the collection.")
    version: Optional[StrictStr] = Field(default=None, description="Version string that uniquely identifies the rules that were applied.")
    required_for_next_transition: Optional[StrictBool] = Field(default=None, description="Indicates if the verifications in the collection need to pass in order to be able to transition the case to the next stage. This field is marking as required the collection itself as a whole, this is not related to outstanding_optional, outstanding_optional is just indicating that the collection has passed verifications, but the collection itself is still required to pass for the next transition.")
    required_for_future_transition: Optional[StrictBool] = Field(default=None, description="Indicates if the verifications in the collection are needed only for future transitions, not the next one. This field is marking as required the collection itself as a whole, this is not related to outstanding_optional, outstanding_optional is just indicating that the collection has passed verifications, but the collection itself is still required to pass for a future transition.")
    outstanding_optional: Optional[StrictBool] = Field(default=None, description="Indicates whether all failed verifications are optional.")
    __properties: ClassVar[List[str]] = ["collection_id", "name", "passed", "verifications", "version", "required_for_next_transition", "required_for_future_transition", "outstanding_optional"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CdmVerificationCollection from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in verifications (list)
        _items = []
        if self.verifications:
            for _item_verifications in self.verifications:
                if _item_verifications:
                    _items.append(_item_verifications.to_dict())
            _dict['verifications'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CdmVerificationCollection from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "collection_id": obj.get("collection_id"),
            "name": obj.get("name"),
            "passed": obj.get("passed"),
            "verifications": [CdmVerification.from_dict(_item) for _item in obj["verifications"]] if obj.get("verifications") is not None else None,
            "version": obj.get("version"),
            "required_for_next_transition": obj.get("required_for_next_transition"),
            "required_for_future_transition": obj.get("required_for_future_transition"),
            "outstanding_optional": obj.get("outstanding_optional")
        })
        return _obj


